import 'dart:async';

import 'package:flutter/material.dart';
import 'package:stream_transform/stream_transform.dart';
import 'view.dart';

class OwlnextAutocomplete extends StatefulWidget {
  /// Owlnext form input
  /// Based on Material design and uses [CompositedTransformTarget]
  /// Used to automate all autocompletes behaviors ine one place
  const OwlnextAutocomplete({
    required this.controller,
    required this.onChange,
    required this.optionsList,
    required this.onSelect,
    this.label,
    this.customOffset = const Offset(0, 75),
    this.maxDropdownHeight = 175,
    this.hintText = '',
    this.suffixIcon,
    this.prefixIcon,
    super.key 
  });

  /// Text input controller
  final TextEditingController controller;
  /// Callback on change text input controller
  final Future<void> Function(String) onChange;
  /// Callback on select option
  final void Function(AutocompleteOption) onSelect;
  /// Options List
  final List<AutocompleteOption> optionsList;
  /// Label above input
  final String? label;
  /// Custom [Offset] for the portal (default [Offset(0, 75)])
  final Offset customOffset;
  /// Maximum dropdown height, default at 175
  final double maxDropdownHeight;
  final String? hintText;
  final Widget? suffixIcon;
  final Widget? prefixIcon;

  @override
  OwlnextAutocompleteState createState() {
    return OwlnextAutocompleteState();
  }
}

class OwlnextAutocompleteState extends State<OwlnextAutocomplete> with ViewMixin {
  final OverlayPortalController portalController = OverlayPortalController();
  StreamController<String> streamController = StreamController();
  LayerLink link = LayerLink();
  FocusNode focus = FocusNode();
  double? inputWidth;
  bool isArrowDown = true;
  bool isLoading = false;
  bool isDropdownShown = false;

  @override
  void initState() {
    streamController.stream.debounce(
      const Duration(milliseconds: 300)).listen((String s) => handleOnChange(s)
    );
    
    super.initState();
    focus.addListener(onFocusChange);
  }

  @override
  void dispose() {
    super.dispose();
    focus.removeListener(onFocusChange);
    focus.dispose();
  }
  
  void onFocusChange() async {
    setState(() {
      inputWidth = context.size?.width;  
    });

    if (focus.hasFocus == true && portalController.isShowing == false) {
      portalController.show();

      setState(() {
        isArrowDown = false;
      });
    } else if (focus.hasFocus == false && portalController.isShowing == true) {
      await Future.delayed(const Duration(milliseconds: 200));
      handleClosePortal();
    }
  }

  void handleClosePortal() async {
    portalController.hide();
    setState(() {
      isArrowDown = true;
    });

    await Future.delayed(const Duration(milliseconds: 100));
    focus.unfocus();
  }

  void onSelect(AutocompleteOption selectedOption) {
    widget.onSelect(selectedOption);
    handleClosePortal();
  }

  void handleOnChange(String value) async {
    if (widget.controller.text.length > 1) {
      setState(() {
        isLoading = true;
        isDropdownShown = true;
      });

      try {
        await widget.onChange(value);
      } catch(e) {
        if (mounted) {
          setState(() {
            isLoading = false;
          });
        }
      }

      if (mounted) {
        setState(() {
          isLoading = false;
        });
      }
    } else {
      setState(() {
        isDropdownShown = false;
      });
    }
  }

  @override
  Widget build(BuildContext context) {
    return render(context, this);
  }
}

class AutocompleteOption {
  final Widget widget;
  final String value;

  AutocompleteOption({
    required this.widget,
    required this.value,
  });
}