// ignore_for_file: unnecessary_null_comparison

import 'package:flutter/material.dart';
import 'package:flutter/services.dart';
import 'package:nirrti/nirrti.dart';
import 'package:nirrti/src/extensions/datetime_extensions.dart';
import 'view.dart';

class OwlnextInput extends StatefulWidget {
  /// Owlnext form input
  /// Based on Material design and uses [TextFormField]
  /// Used to automate all input behavior in one place
  /// 
  /// Can be customized through a variety of properties, like type
  /// [TextInputSubType] for specific behaviors
  const OwlnextInput({
    super.key, 
    this.type = TextInputSubType.text,
    this.controller,
    this.label,
    this.errorText = '',
    this.minimumLines = 1,
    this.hintText = '',
    this.suffixIcon,
    this.prefixIcon,
    this.enabled = true,
    this.onChanged,
    this.suffixText,
    this.charsLimit,
    this.isMandatory = false,
    this.infoCallback,
    this.amountMaxLimit,
    this.decimalsMaxLimit,
    this.numberMaxLimit,
    this.numberMinLimit,
    this.focus,
    this.customRegex,
    this.minimumDate,
    this.maximumDate,
    this.onSubmitted,
    this.isInsideList = true,
    this.text,
    this.axis = Axis.vertical,
    this.hasCopyToClipboard = false,
    this.textInputAction,
  }) :
    assert(amountMaxLimit == null || (amountMaxLimit != null && type == TextInputSubType.amount)),
    assert(decimalsMaxLimit == null || (decimalsMaxLimit != null && type == TextInputSubType.amount)),
    assert(numberMaxLimit == null || (numberMaxLimit != null && type == TextInputSubType.numbers)),
    assert(numberMinLimit == null || (numberMinLimit != null && type == TextInputSubType.numbers)),
    assert(minimumDate == null || (minimumDate != null && type == TextInputSubType.date)),
    assert(maximumDate == null || (maximumDate != null && type == TextInputSubType.date)),
    assert((text == null && controller != null) || (text != null && controller == null));

  
  /// Text input type, determines behavior depending on type
  final TextInputSubType type;
  /// Text input controller
  final TextEditingController? controller;
  /// Is enabled or not
  final bool enabled;
  /// Is mandatory or not, determines the validator behavior
  final bool isMandatory;
  /// Is the text input inside a list, enables the autoscroll on validation
  final bool isInsideList;
  /// Called every time the text input is changed, with the current String as arg
  final void Function(String)? onChanged;
  /// Number of characters limitation, null if none
  final int? charsLimit;
  /// Number of lines for text areas, null if classic input
  final int? minimumLines;
  /// Called when clicking on the information icon, right to the label
  /// Can be anything (openPanel, openPopup...)
  final Function? infoCallback;

  /// Specifies the maximum amount if the input is an amount
  final double? amountMaxLimit;
  /// Specifies the maximum number of decimals, if the input is an amount
  final int? decimalsMaxLimit;
  /// Specifies the maximum number to reach if the input is a number
  final int? numberMaxLimit;
  /// Specifies the minimum number to reach if the input is a number
  final int? numberMinLimit;
  /// Specifies a custom FocusNode for the input
  /// Useful if there is a custom Focus-related behavior
  final FocusNode? focus;
  /// Specifies a custom Regex to check on validation
  final RegExp? customRegex;
  /// Specifies a minimum date to check on validation, if the input is a date
  final DateTime? minimumDate;
  /// Specifies a maximum date to check on validation, if the input is a date
  final DateTime? maximumDate;
  /// Custom callback on input validation
  /// Useful for calling a specific API for example (BAN, SIRET...)
  final void Function(String)? onSubmitted;
  final String? label;
  final String? errorText;
  final String? hintText;
  final Widget? suffixIcon;
  final Widget? prefixIcon;
  final String? suffixText;
  /// Text string to not use a controller
  /// Can be used alongside [onChanged]
  final String? text;
  /// Set the axis for the label -> input
  final Axis? axis;
  /// Specifies if the copy-to-clipboard button should be here
  final bool hasCopyToClipboard;
  /// Determines the icon for validation in the virtual keyboard
  final TextInputAction? textInputAction;

  @override
  OwlnextInputState createState() {
    return OwlnextInputState();
  }
}

class OwlnextInputState extends State<OwlnextInput> with ViewMixin {
  FocusNode focusNode = FocusNode();

  @override
  void initState() {
    super.initState();

    WidgetsBinding.instance.addPostFrameCallback((_) {
      if (widget.focus != null) {
        setState(() {
          focusNode = widget.focus!;
        });
      }
    });
  }

  @override
  void dispose() {
    super.dispose();
  }

  TextStyle? get getLabelStyle {
    return Artist().labelTextStyle;
  }

  /// Get the isDense parameter, retreives the one on the Artist() instance if no override
  bool get getIsDense {
    if (Artist().inputDecoration.isDense != null) {
      return Artist().inputDecoration.isDense!;
    } else {
      return true;
    }
  }

  /// Get the suffix icon
  /// If no override, displays the suffix icon according to the input type
  Widget? get getSuffixIcon {
    if ([TextInputSubType.date, TextInputSubType.shortDate].contains(widget.type)) {
      return InkWell(
        onTap: () async {
          DateTime? pickedDate = await showDatePicker(
            context: context,
            initialDate: DateTime.now(),
            firstDate: DateTime(2000),
            lastDate: DateTime(2101),
            builder: (context, child) {
              return Theme(
                data: Theme.of(context).copyWith(
                  colorScheme: ColorScheme.light(
                    primary: Theme.of(context).colorScheme.primary,
                    onPrimary: Colors.white,
                    onSurface: Colors.black,
                  ),
                  textButtonTheme: TextButtonThemeData(
                    style: TextButton.styleFrom(
                      foregroundColor: Colors.black, // button text color
                    ),
                  ),
                ),
                child: child!,
              );
            },
          );
      
          if(pickedDate != null){
            if (widget.type == TextInputSubType.shortDate) {
              if (widget.controller != null) {
                widget.controller!.text = pickedDate.formatShortDateFromDateTime();
              } else {
                widget.onChanged!(pickedDate.formatShortDateFromDateTime());
              }
            } else {
              if (widget.controller != null) {
                widget.controller!.text = pickedDate.formatDateFromDateTime();
              } else {
                widget.onChanged!(pickedDate.formatDateFromDateTime());
              }
            }
          }
        },
        child: Icon(
          Icons.date_range, 
          color: Colors.black.withOpacity(0.5)
        ),
      );
    } else if (widget.type == TextInputSubType.amount && widget.suffixText == null) {
      return const Padding(
        padding: EdgeInsets.only(top: 8),
        child: Text('€')
      );
    } else if (widget.suffixIcon != null) {
      return widget.suffixIcon;
    } else {
      return Padding(
        padding: const EdgeInsets.fromLTRB(0, 12, 16, 0),
        child: Text(
          widget.suffixText ?? '',
          textAlign: TextAlign.end,
          style: TextStyle(
            fontSize: 13,
            color: Colors.black.withOpacity(0.5)
          )
        ),
      );
    }
  }

  // Handles the copy to clipboard feature
  void handleCopyToClipBoard() async {
    await Clipboard.setData(ClipboardData(text: widget.controller != null ? widget.controller!.text : widget.text!));

    if (mounted) {
      ScaffoldMessenger.of(context).showSnackBar(
        SnackBar(
          duration: const Duration(seconds: 1),
          backgroundColor: Colors.transparent,
          behavior: SnackBarBehavior.floating,
          elevation: 0,
          content: Align(
            alignment: Alignment.centerRight,
            child: Container(
              padding: const EdgeInsets.all(15),
              decoration: BoxDecoration(
                color: const Color.fromARGB(255, 152, 152, 152),
                borderRadius: BorderRadius.circular(10)
              ),
              width: 200,
              child: const Text('Texte copié')
            ),
          )
        )
      );
    }
  }

  @override
  Widget build(BuildContext context) {
    return render(context, this);
  }
}

/// All subtypes of text input
/// Determines the input's behavior on change and on form validation
enum TextInputSubType {
  text,
  date,
  shortDate,
  numbers,
  amount,
  email,
  hours,
  password,
}