


import 'package:flutter/material.dart';
import 'view.dart';

class OwlnextTable extends StatefulWidget {
  /// Owlnext Table component
  /// Handles columns and rows, lazyloading, search and filters
  /// 
  /// Check the individual props to check the process
  const OwlnextTable({
    super.key,
    required this.headersList,
    required this.rowsList,
    this.callback,
    this.hasSearch = true,
    this.onClickRow,
    this.button,
    this.isScrollable = true,
    required this.tableKey,
    this.filters = const [],
    this.hasCheckboxes = false,
  });

  /// Headers to show in the table
  final List<DataColumn> headersList;
  /// Individual rows
  /// Can have custom widgets list, but must have the same length as the headers
  final List<CustomTableRow> rowsList;
  /// Callback function on search, filters clicks and lazyLoading
  final Future<bool> Function({ String? search, FilterCall? filter, int? pageNumber })? callback;
  /// Determines if the table has a searchbar
  final bool hasSearch;
  /// Callback on click row
  final Function? onClickRow;
  /// Determines if the table has a top right button
  /// Usually a "+" button to trigger a post callback
  final Function? button;
  /// Determines if the table has to be scrollable, or must have the same length as its childs
  final bool isScrollable;
  /// Re-render table key
  final UniqueKey tableKey;
  /// Filters list
  /// Does not display if list is empty
  final List<CustomFilter> filters;
  /// Determines if the table has checkboxes
  final bool hasCheckboxes;

  @override
  OwlnextTableState createState() {
    return OwlnextTableState();
  }
}

class OwlnextTableState extends State<OwlnextTable> with ViewMixin {
  List<Widget> headersList = [];
  List<CustomTableRow> rowsList = [];
  CustomFilter? currentActiveFilter;
  TextEditingController searchController = TextEditingController();
  ScrollController scrollController = ScrollController();
  bool alreadyLoadingNextPage = false;
  bool isLoading = false;
  bool isEnding = false;
  int pageNumber = 1;

  @override
  void initState() {
    super.initState();

    WidgetsBinding.instance.addPostFrameCallback((_) {
      initCall();
    });
  }

  @override
  void dispose() {
    super.dispose();
    searchController.dispose();
    scrollController.dispose();
  }

  /// First call to populate the table
  void initCall() async {
    if (widget.callback != null) {
      setState(() {
        isLoading = true;
      });

      await widget.callback!(search: '', filter: null, pageNumber: 1);

      if (mounted) {
        setState(() {
          isLoading = false;
        });
      }
    }
  }

  /// Triggers when the scroll get to 80% of the scrollview
  /// Get the callback response
  void handleScrollTriggerNextPage(ScrollUpdateNotification notification) async {
    double threshold = notification.metrics.maxScrollExtent - (notification.metrics.maxScrollExtent / 100 * 15);
    
    if (notification.metrics.pixels > threshold && alreadyLoadingNextPage == false && isEnding == false) {
      alreadyLoadingNextPage = true;
      bool result = await widget.callback!(search: searchController.text, filter: null, pageNumber: pageNumber + 1);

      setState(() {
        isEnding = result;
      });
      
      pageNumber += 1;
      await Future.delayed(const Duration(milliseconds: 50));
      alreadyLoadingNextPage = false;
    }
  }

  void handleCallbackTrigger(String value) {
    setState(() {
      currentActiveFilter = null;
    });

    if (widget.callback != null) {
      widget.callback!(search: value);
    }
  }

  /// Activate / deactivate a clicked filter depending on its state
  void handleChangeCurrentActiveFilter(CustomFilter? newActiveFilter) async {
    setState(() {
      isLoading = true;
    });
    
    if (newActiveFilter == null || currentActiveFilter?.label == newActiveFilter.label) {
      setState(() {
        currentActiveFilter = null;
      });

      if (widget.callback != null) {
        await widget.callback!(
          search: searchController.text, 
          filter: null, 
          pageNumber: 1,
        );
      }
    } else {
      setState(() {
        currentActiveFilter = newActiveFilter;
      });

      if (widget.callback != null) {
        await widget.callback!(
          search: searchController.text, 
          filter: FilterCall(
            label: newActiveFilter.filterName, 
            value: newActiveFilter.value
          ), 
          pageNumber: 1,
        );
      }
    }

    setState(() {
      isLoading = false;
    });
  }
  
  @override
  Widget build(BuildContext context) {
    return render(context, this);
  }
}


class CustomTableRow{
  dynamic item;
  List<Widget> widgets;
  bool? selected;

  CustomTableRow({
    required this.item,
    required this.widgets,
    this.selected,
  });
}

class CustomFilter {
  String label;
  String value;
  String filterName;
  Color color;

  CustomFilter({
    required this.label,
    required this.value,
    required this.filterName,
    required this.color,
  });
}

class FilterCall {
  String label;
  String value;

  FilterCall({
    required this.label,
    required this.value,
  });

  String generateFilterCall() {
    return '$label=$value';
  }
}